/* ------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2009-2014

   Project:  Eloquence JDLG Plugin Interface
   File:     Axis.java
   Function: JDLG Charts Plugin: abstract Line/Bar base class
   $Id: Axis.java,v 1.8 2014/12/11 12:17:23 rhg Exp $

   Edit history:
   2009-04-06, ap   - creation
   2009-04-28, mike - finalized
   2014-12-03, rhg  - revised, enhancements, bugfixes, code cleanup
 ------------------------------------------------------------------------ */

package com.eloquence.plugin.charts;

import com.eloquence.api.*;
import com.eloquence.services.tools.*;

import java.awt.*;
import java.util.*;

import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.data.category.*;

public abstract class Axis
   extends Chart
   implements Plugin
{
   protected ChartPanel peerPanel;
   protected DefaultCategoryDataset dataset = new DefaultCategoryDataset();
   protected boolean datasetModified = true;

   // ------------------------------------------------------------------------
   // Plugin
   // ------------------------------------------------------------------------

   public void onDispose()
   {
      peerPanel = null;
      dataset = null;
   }

   public void onEloq2Swing()
   {
      if (datasetModified)
      {
         dataset.clear();

         int numRows = rows.size();
         int numColumns = columns.size();

         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            Row row = (Row)rows.get(rowIndex);
            Vector rowContent = (Vector)content.get(rowIndex);

            for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
            {
               dataset.addValue(
                  ((Double)rowContent.get(columnIndex)).doubleValue(),
                  row.getLabel(),
                  (String)columns.get(columnIndex));
            }
         }
      }

      if (attributeModified)
      {
         JFreeChart chart = peerPanel.getChart();
         CategoryPlot plot = (CategoryPlot) chart.getPlot();
         plot.getDomainAxis().setLabel(columnAxis);
         plot.getRangeAxis().setLabel(valueAxis);
         plot.setRangeGridlinePaint(gridColor);
         plot.setOrientation(horizontal ? PlotOrientation.HORIZONTAL
                                        : PlotOrientation.VERTICAL);
      }
   }

   // ------------------------------------------------------------------------
   // get/set columnaxis
   // ------------------------------------------------------------------------

   private static final String COLUMNAXIS_ATTR_NAME = "columnaxis";
   // alternative attribute name to retain backward compatibiliy
   private static final String VAXIS_ATTR_NAME = "vaxis";
   private String columnAxis = "";

   private final void setColumnAxis(String columnAxis)
   {
      this.columnAxis = columnAxis;
      attributeModified = true;
   }

   private final String getColumnAxis()
   {
      return columnAxis;
   }

   // ------------------------------------------------------------------------
   // get/set valueaxis
   // ------------------------------------------------------------------------

   private static final String VALUEAXIS_ATTR_NAME = "valueaxis";
   // alternative attribute name to retain backward compatibiliy
   private static final String HAXIS_ATTR_NAME = "haxis";
   private String valueAxis = "";

   private final void setValueAxis(String valueAxis)
   {
      this.valueAxis = valueAxis;
      attributeModified = true;
   }

   private final String getValueAxis()
   {
      return valueAxis;
   }

   // ------------------------------------------------------------------------
   // get/set orientation
   // ------------------------------------------------------------------------

   private static final String ORIENTATION_ATTR_NAME = "orientation";
   private boolean horizontal = false;

   private final void setOrientation(int horizontal)
   {
      this.horizontal = horizontal != 0;
      attributeModified = true;
   }

   private final int getOrientation()
   {
      return horizontal ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set gridcolor
   // ------------------------------------------------------------------------

   private static final String GRIDCOLOR_ATTR_NAME = "gridcolor";
   private static final Color defaultGridColor = new Color(0, 0, 0);
   private Color gridColor = defaultGridColor;

   private final boolean setGridColor(String colorString)
   {
      colorString = colorString.trim();
      if (colorString.length() == 0)
         gridColor = defaultGridColor;
      else {
         int idx = colorString.indexOf('#');
         if (idx == -1)
            return false;

         int colorRgb;
         try
         {
            colorRgb = Integer.parseInt(
               colorString.substring(idx + 1), 16);
         }
         catch (NumberFormatException e)
         {
            return false;
         }

         gridColor = new Color(colorRgb);
      }

      attributeModified = true;
      return true;
   }

   private final String getGridColor()
   {
      return '#' + HexTool.int2hexString(gridColor.getRGB(), 3);
   }

   // ------------------------------------------------------------------------
   // set clear
   // ------------------------------------------------------------------------

   private static final String CLEAR_ATTR_NAME = "clear";

   private final void setClear()
   {
      content.clear();
      rows.clear();
      columns.clear();
      datasetModified = true;
   }

   // ------------------------------------------------------------------------
   // get/set content
   // ------------------------------------------------------------------------

   private static final String CONTENT_ATTR_NAME = "content";
   private Vector content = new Vector();

   private final boolean parseLine(String line)
   {
      int numColumns = columns.size();
      if (numColumns == 0)
      {
         StringTokenizer columnTokenizer = new StringTokenizer(line, delim);
         while (columnTokenizer.hasMoreTokens())
            columns.add(columnTokenizer.nextToken());

         return true;
      }

      StringTokenizer rowTokenizer = new StringTokenizer(line, delim);
      if (rowTokenizer.hasMoreTokens())
      {
         String label = rowTokenizer.nextToken();
         Vector rowContent = new Vector();

         for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
         {
            double value = 0;
            if (rowTokenizer.hasMoreTokens())
            {
               try
               {
                  value = new Double(rowTokenizer.nextToken()).doubleValue();
               }
               catch (NumberFormatException e)
               {
                  return false;
               }
            }

            rowContent.add(new Double(value));
         }

         content.add(rowContent);

         Color barColor = null;
         if (rowTokenizer.hasMoreTokens())
         {
            String colorString = rowTokenizer.nextToken().trim();
            int idx = colorString.indexOf('#');
            if (idx == -1)
               return false;

            int color;
            try
            {
               color = Integer.parseInt(
                  colorString.substring(idx + 1), 16);
            }
            catch (NumberFormatException e)
            {
               return false;
            }

            barColor = new Color(color);
         }

         rows.add(new Row(label, barColor));
      }

      return true;
   }

   private final boolean setContent(String contentString)
   {
      content.clear();
      rows.clear();
      columns.clear();
      datasetModified = true;

      StringTokenizer lineTokenizer = new StringTokenizer(contentString, "\n");
      while (lineTokenizer.hasMoreTokens())
         if (!parseLine(lineTokenizer.nextToken()))
            return false;

      return true;
   }

   private final String getContent()
   {
      int numColumns = columns.size();
      if (numColumns == 0)
         return "";

      String contentString = "";

      for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
      {
         if (columnIndex > 0)
            contentString = contentString.concat(delim);
         contentString = contentString.concat((String)columns.get(columnIndex));
      }

      contentString = contentString.concat("\n");

      int numRows = rows.size();
      for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
      {
         Row row = (Row)rows.get(rowIndex);
         Vector rowContent = (Vector)content.get(rowIndex);

         contentString = contentString.concat(row.getLabel());

         for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
            contentString = contentString.concat(
               delim + formatValue((Double)rowContent.get(columnIndex)));

         Color rowColor = row.getColor();
         if (rowColor != null)
            contentString = contentString.concat(
               delim + '#' + HexTool.int2hexString(rowColor.getRGB(), 3));

         contentString = contentString.concat("\n");
      }

      return contentString;
   }

   // ------------------------------------------------------------------------
   // set add
   // ------------------------------------------------------------------------

   private static final String ADD_ATTR_NAME = "add";

   private final boolean setAdd(String line)
   {
      if (parseLine(line))
      {
         datasetModified = true;
         return true;
      }

      return false;
   }

   // ------------------------------------------------------------------------
   // get/set column
   // ------------------------------------------------------------------------

   private static final String COLUMN_ATTR_NAME = "column";
   protected Vector columns = new Vector();

   private final boolean setColumn(String label, int columnIndex)
   {
      if (columnIndex < 0 || columnIndex >= columns.size())
         return false;

      columns.set(columnIndex, label);
      datasetModified = true;
      return true;
   }

   private final String getColumn(int columnIndex)
   {
      if (columnIndex >= 0 && columnIndex < columns.size())
         return (String)columns.get(columnIndex);
      return "";
   }

   // ------------------------------------------------------------------------
   // get/set columns
   // ------------------------------------------------------------------------

   private static final String COLUMNS_ATTR_NAME = "columns";

   private final void setColumns(int numColumns)
   {
      columns.clear();
      datasetModified = true;

      int numRows = rows.size();
      if (numColumns < 1)
      {
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            Vector rowContent = (Vector)content.get(rowIndex);
            rowContent.clear();
         }

         return;
      }

      for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
      {
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            Vector rowContent = (Vector)content.get(rowIndex);
            if (columnIndex == 0)
               rowContent.clear();

            rowContent.add(new Double(0));
         }

         columns.add(new String("Column " + columnIndex));
      }
   }

   private final int getColumns()
   {
      return columns.size();
   }

   // ------------------------------------------------------------------------
   // get/set row
   // ------------------------------------------------------------------------

   private static final String ROW_ATTR_NAME = "row";
   protected Vector rows = new Vector();

   private final boolean setRow(String labelWithColor, int rowIndex)
   {
      if (rowIndex < 0 || rowIndex >= rows.size())
         return false;

      int separator = labelWithColor.indexOf(delim);
      if (separator == -1)
         rows.set(rowIndex, new Row(labelWithColor));
      else {
         String colorString = labelWithColor.substring(separator + 1).trim();
         Color color = null;
         if (colorString.length() > 0)
         {
            int idx = colorString.indexOf('#');
            if (idx == -1)
               return false;

            int colorRgb;
            try
            {
               colorRgb = Integer.parseInt(
                  colorString.substring(idx + 1), 16);
            }
            catch (NumberFormatException e)
            {
               return false;
            }

            color = new Color(colorRgb);
         }

         rows.set(rowIndex,
                  new Row(labelWithColor.substring(0, separator),
                  color));
      }

      datasetModified = true;
      return true;
   }

   private final String getRow(int rowIndex)
   {
      if (rowIndex >= 0 && rowIndex < rows.size())
         return ((Row)rows.get(rowIndex)).getLabel();
      return "";
   }

   // ------------------------------------------------------------------------
   // get/set rows
   // ------------------------------------------------------------------------

   private static final String ROWS_ATTR_NAME = "rows";

   private final void setRows(int numRows)
   {
      content.clear();
      rows.clear();
      datasetModified = true;

      int numColumns = columns.size();
      for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
      {
         Vector rowContent = new Vector();
         for (int column = 0; column < numColumns; column++)
            rowContent.add(new Double(0));

         content.add(rowContent);
         rows.add(new Row("Row " + rowIndex));
      }
   }

   private final int getRows()
   {
      return rows.size();
   }

   // ------------------------------------------------------------------------
   // get/set value[][]
   // ------------------------------------------------------------------------

   private static final String VALUE_ATTR_NAME = "value";

   private final boolean setValue(String value, int rowIndex, int columnIndex)
   {
      if (rowIndex < 0 || rowIndex >= rows.size())
         return false;

      if (columnIndex < 0 || columnIndex >= columns.size())
         return false;

      try
      {
         Vector rowContent = (Vector)content.get(rowIndex);
         rowContent.set(columnIndex, new Double(value));
         datasetModified = true;
      }
      catch (NumberFormatException e)
      {
         return false;
      }

      return true;
   }

   private final String getValue(int rowIndex, int columnIndex)
   {
      if (rowIndex >= 0 && rowIndex < rows.size())
         if (columnIndex >= 0 && columnIndex < columns.size())
         {
            Vector rowContent = (Vector)content.get(rowIndex);
            return formatValue((Double)rowContent.get(columnIndex));
         }

      return "";
   }

   // ------------------------------------------------------------------------
   // attributes
   // ------------------------------------------------------------------------

   private static final int COLUMNAXIS_ATTR_ID = 1;
   private static final int VALUEAXIS_ATTR_ID = 2;
   private static final int ORIENTATION_ATTR_ID = 3;
   private static final int GRIDCOLOR_ATTR_ID = 4;
   private static final int CLEAR_ATTR_ID = 5;
   private static final int CONTENT_ATTR_ID = 6;
   private static final int ADD_ATTR_ID = 7;
   private static final int COLUMN_ATTR_ID = 8;
   private static final int COLUMNS_ATTR_ID = 9;
   private static final int ROW_ATTR_ID = 10;
   private static final int ROWS_ATTR_ID = 11;
   private static final int VALUE_ATTR_ID = 12;

   public void onClone(Plugin clone)
   {
      super.onClone(clone);

      Axis cloneObj = (Axis) clone;
      cloneObj.setColumnAxis(getColumnAxis());
      cloneObj.setValueAxis(getValueAxis());
      cloneObj.setOrientation(getOrientation());
      cloneObj.setGridColor(getGridColor());

      cloneObj.content.clear();
      cloneObj.rows.clear();
      cloneObj.columns.clear();
      cloneObj.datasetModified = true;

      int numRows = rows.size();
      int numColumns = columns.size();
      for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
      {
         Vector rowContent = (Vector)content.get(rowIndex);
         Vector cloneRowContent = new Vector();

         for (int columnIndex = 0; columnIndex < numColumns; columnIndex++)
         {
            if (rowIndex == 0)
               cloneObj.columns.add(
                  new String((String)columns.get(columnIndex)));

            cloneRowContent.add(new Double(
               ((Double)rowContent.get(columnIndex)).doubleValue()));
         }

         cloneObj.content.add(cloneRowContent);
         cloneObj.rows.add(new Row((Row)rows.get(rowIndex)));
      }
   }

   public boolean onSet(String attribute, java.util.List index,
                        UnifiedData data)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case COLUMNAXIS_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setColumnAxis(data.getStringData());
               break;

            case VALUEAXIS_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setValueAxis(data.getStringData());
               break;

            case ORIENTATION_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setOrientation(data.getIntData());
               break;

            case GRIDCOLOR_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               return setGridColor(data.getStringData());

            case CLEAR_ATTR_ID:
               if (indexElements != 0)
                  return false;
               setClear();
               break;

            case CONTENT_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               return setContent(data.getStringData());

            case ADD_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               return setAdd(data.getStringData());

            case COLUMN_ATTR_ID:
            {
               if (indexElements != 1)
                  return false;
               if (!data.isStringData())
                  return false;
               UnifiedData columnIndex = (UnifiedData)index.get(0);
               if (columnIndex.isStringData())
	          return false;
               return setColumn(data.getStringData(),
                                columnIndex.getIntData());
            }

            case COLUMNS_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setColumns(data.getIntData());
               break;

            case ROW_ATTR_ID:
            {
               if (indexElements != 1)
                  return false;
               if (!data.isStringData())
                  return false;
               UnifiedData rowIndex = (UnifiedData)index.get(0);
               if (rowIndex.isStringData())
	          return false;
               return setRow(data.getStringData(),
                             rowIndex.getIntData());
            }

            case ROWS_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setRows(data.getIntData());
               break;

            case VALUE_ATTR_ID:
            {
               if (indexElements != 2)
                  return false;
               if (!data.isStringData())
                  return false;
               UnifiedData rowIndex = (UnifiedData)index.get(0);
               if (rowIndex.isStringData())
	          return false;
               UnifiedData columnIndex = (UnifiedData)index.get(1);
               if (columnIndex.isStringData())
	          return false;
               return setValue(data.getStringData(),
                               rowIndex.getIntData(),
                               columnIndex.getIntData());
            }
         }

         return true;
      }

      return super.onSet(attribute, index, data);
   }

   public UnifiedData onGet(String attribute, java.util.List index)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case COLUMNAXIS_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getColumnAxis());

            case VALUEAXIS_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getValueAxis());

            case ORIENTATION_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getOrientation());

            case GRIDCOLOR_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getGridColor());

            case CONTENT_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getContent());

            case COLUMN_ATTR_ID:
            {
               if (indexElements != 1)
                  return null;
               UnifiedData columnIndex = (UnifiedData)index.get(0);
               if (columnIndex.isStringData())
	          return null;
               return new UnifiedData(getColumn(columnIndex.getIntData()));
            }

            case COLUMNS_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getColumns());

            case ROW_ATTR_ID:
            {
               if (indexElements != 1)
                  return null;
               UnifiedData rowIndex = (UnifiedData)index.get(0);
               if (rowIndex.isStringData())
	          return null;
               return new UnifiedData(getRow(rowIndex.getIntData()));
            }

            case ROWS_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getRows());

            case VALUE_ATTR_ID:
            {
               if (indexElements != 2)
                  return null;
               UnifiedData rowIndex = (UnifiedData)index.get(0);
               if (rowIndex.isStringData())
	          return null;
               UnifiedData columnIndex = (UnifiedData)index.get(1);
               if (columnIndex.isStringData())
	          return null;
               return new UnifiedData(getValue(rowIndex.getIntData(),
                                               columnIndex.getIntData()));
            }
         }
      }

      return super.onGet(attribute, index);
   }

   private static HashMap attributeList = new HashMap();
   static
   {
      attributeList.put(COLUMNAXIS_ATTR_NAME, new Integer(COLUMNAXIS_ATTR_ID));
      attributeList.put(VAXIS_ATTR_NAME, new Integer(COLUMNAXIS_ATTR_ID));
      attributeList.put(VALUEAXIS_ATTR_NAME, new Integer(VALUEAXIS_ATTR_ID));
      attributeList.put(HAXIS_ATTR_NAME, new Integer(VALUEAXIS_ATTR_ID));
      attributeList.put(ORIENTATION_ATTR_NAME, new Integer(ORIENTATION_ATTR_ID));
      attributeList.put(GRIDCOLOR_ATTR_NAME, new Integer(GRIDCOLOR_ATTR_ID));
      attributeList.put(CLEAR_ATTR_NAME, new Integer(CLEAR_ATTR_ID));
      attributeList.put(CONTENT_ATTR_NAME, new Integer(CONTENT_ATTR_ID));
      attributeList.put(ADD_ATTR_NAME, new Integer(ADD_ATTR_ID));
      attributeList.put(COLUMN_ATTR_NAME, new Integer(COLUMN_ATTR_ID));
      attributeList.put(COLUMNS_ATTR_NAME, new Integer(COLUMNS_ATTR_ID));
      attributeList.put(ROW_ATTR_NAME, new Integer(ROW_ATTR_ID));
      attributeList.put(ROWS_ATTR_NAME, new Integer(ROWS_ATTR_ID));
      attributeList.put(VALUE_ATTR_NAME, new Integer(VALUE_ATTR_ID));
   }

   // ------------------------------------------------------------------------
   // Row, represents one data row in the chart (label and color)
   // ------------------------------------------------------------------------

   protected class Row
   {
      private String label;
      private Color color;

      public Row(String label)
      {
         this.label = label;
         this.color = null;
      }

      public Row(String label, Color color)
      {
         this.label = label;
         this.color = color;
      }

      public Row(Row other)
      {
         label = new String(other.label);
         if (other.color != null)
            color = new Color(other.color.getRGB());
         else
            color = null;
      }

      public final void setLabel(String label)
      {
         this.label = label;
      }

      public final String getLabel()
      {
         return label;
      }

      public final void setColor(Color color)
      {
         this.color = color;
      }

      public final Color getColor()
      {
         return color;
      }
   }
}
