/* ------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2009-2014

   Project:  Eloquence JDLG Plugin Interface
   File:     Chart.java
   Function: JDLG Charts Plugin: abstract base class
   $Id: Chart.java,v 1.10 2014/12/11 12:17:23 rhg Exp $

   Edit history:
   2009-04-20, ap   - creation
   2009-09-25, rhg  - finalized
   2014-04-08, rhg  - api version 1.1 additions
   2014-12-03, rhg  - revised, enhancements, bugfixes, code cleanup
 ------------------------------------------------------------------------ */

package com.eloquence.plugin.charts;

import com.eloquence.api.*;
import com.eloquence.services.tools.*;

import java.awt.*;
import java.util.*;
import java.text.*;

import org.jfree.ui.*;

public abstract class Chart
   implements Plugin
{
   protected boolean chartModified = true;
   protected boolean attributeModified = true;

   // ------------------------------------------------------------------------
   // Plugin
   // ------------------------------------------------------------------------

   public void onSwing2Eloq()
   {
   }

   public void onFontModified(Font font)
   {
   }

   public boolean isFocusable()
   {
      return false;
   }

   public void onFocusReceived()
   {
   }

   public void onFocusLost()
   {
   }

   public Component onGetFocusPeer()
   {
      return null;
   }

   public void onTabFocus(boolean toNext)
   {
   }

   public boolean fireRuleOnLosingFocus()
   {
      return false;
   }

   public boolean onRuleOverride()
   {
      return false;
   }

   public boolean allowRMBR()
   {
      return false;
   }

   // ------------------------------------------------------------------------
   // value formatting
   // ------------------------------------------------------------------------

   protected static final String formatValue(double d)
   {
      String value = Double.toString(d);
      if (value.endsWith(".0"))
         return value.substring(0, value.length() - 2);
      return value;
   }

   protected static final String formatValue(Double d)
   {
      String value = d.toString();
      if (value.endsWith(".0"))
         return value.substring(0, value.length() - 2);
      return value;
   }

   // ------------------------------------------------------------------------
   // get/set title
   // ------------------------------------------------------------------------

   private static final String TITLE_ATTR_NAME = "title";
   protected String title = "";

   private final void setTitle(String title)
   {
      this.title = title;
      attributeModified = true;
   }

   private final String getTitle()
   {
      return title;
   }

   // ------------------------------------------------------------------------
   // get/set delim
   // ------------------------------------------------------------------------

   private static final String DELIM_ATTR_NAME = "delim";
   protected String delim = "\t";

   private final void setDelim(String delim)
   {
      this.delim = delim;
   }

   private final String getDelim()
   {
      return delim;
   }

   // ------------------------------------------------------------------------
   // get/set chartbgc
   // ------------------------------------------------------------------------

   private static final String CHARTBGC_ATTR_NAME = "chartbgc";
   private static final Color defaultChartBgc = new Color(255, 255, 255);
   protected Color chartBgc = defaultChartBgc;

   private final boolean setChartBgc(String colorString)
   {
      colorString = colorString.trim();
      if (colorString.length() == 0)
         chartBgc = defaultChartBgc;
      else {
         int idx = colorString.indexOf('#');
         if (idx == -1)
            return false;

         int colorRgb;
         try
         {
            colorRgb = Integer.parseInt(
               colorString.substring(idx + 1), 16);
         }
         catch (NumberFormatException e)
         {
            return false;
         }

         chartBgc = new Color(colorRgb);
      }

      attributeModified = true;
      return true;
   }

   private final String getChartBgc()
   {
      return '#' + HexTool.int2hexString(chartBgc.getRGB(), 3);
   }

   // ------------------------------------------------------------------------
   // get/set fgalpha
   // ------------------------------------------------------------------------

   private static final String FGALPHA_ATTR_NAME = "fgalpha";
   protected int fgAlpha = 100;

   private final void setFgAlpha(int alpha)
   {
      if (alpha < 0)
         alpha = 0;
      else if (alpha > 100)
         alpha = 100;

      fgAlpha = alpha;
      attributeModified = true;
   }

   private final int getFgAlpha()
   {
      return fgAlpha;
   }

   // ------------------------------------------------------------------------
   // get/set border
   // ------------------------------------------------------------------------

   private static final String BORDER_ATTR_NAME = "border";
   protected boolean border = false;

   private static final RectangleInsets insetsNoBorder =
      new RectangleInsets(1, 1, 1, 1);
   private static final RectangleInsets insetsBorder =
      new RectangleInsets(3, 3, 3, 3);
   protected RectangleInsets borderInsets = insetsNoBorder;

   private final void setBorder(int border)
   {
      if (border != 0)
      {
         this.border = true;
         borderInsets = insetsBorder;
      }
      else {
         this.border = false;
         borderInsets = insetsNoBorder;
      }

      attributeModified = true;
   }

   private final int getBorder()
   {
      return border ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set legend
   // ------------------------------------------------------------------------

   private static final String LEGEND_ATTR_NAME = "legend";
   protected boolean legend = true;

   private final void setLegend(int legend)
   {
      this.legend = (legend != 0);
      chartModified = true;
   }

   private final int getLegend()
   {
      return legend ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set valueformat
   // ------------------------------------------------------------------------

   private static final String VALUEFORMAT_ATTR_NAME = "valueformat";
   private DecimalFormat valueFormat = null;
   private DecimalFormat percentFormat = null;

   private final void setValueFormat(String formatString)
   {
      if (formatString.length() > 0)
         valueFormat = new DecimalFormat(formatString);
      valueFormat = null;
   }

   private final String getValueFormat()
   {
      return valueFormat().toPattern();
   }

   protected final DecimalFormat valueFormat()
   {
      if (valueFormat == null)
         valueFormat = new DecimalFormat("0.###");
      return valueFormat;
   }

   protected final DecimalFormat percentFormat()
   {
      if (percentFormat == null)
         percentFormat = new DecimalFormat("0.#%");
      return percentFormat;
   }

   // ------------------------------------------------------------------------
   // get/set label
   // ------------------------------------------------------------------------

   private static final String LABEL_ATTR_NAME = "label";
   protected boolean showLabels = true;

   private final void setLabel(int label)
   {
      showLabels = label != 0;
      attributeModified = true;
   }

   private final int getLabel()
   {
      return showLabels ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set labelformat
   // ------------------------------------------------------------------------

   private static final String LABELFORMAT_ATTR_NAME = "labelformat";
   private String labelFormat = null;

   private final void setLabelFormat(String formatString)
   {
      labelFormat = formatString.length() > 0 ? formatString
                                              : getDefaultLabelFormat();
      attributeModified = true;
   }

   protected final String getLabelFormat()
   {
      if (labelFormat == null)
         labelFormat = getDefaultLabelFormat();
      return labelFormat;
   }

   protected String getDefaultLabelFormat()
   {
      return "{2}";
   }

   // ------------------------------------------------------------------------
   // attributes
   // ------------------------------------------------------------------------

   private static final int TITLE_ATTR_ID = 1;
   private static final int DELIM_ATTR_ID = 2;
   private static final int CHARTBGC_ATTR_ID = 3;
   private static final int FGALPHA_ATTR_ID = 4;
   private static final int BORDER_ATTR_ID = 5;
   private static final int LEGEND_ATTR_ID = 6;
   private static final int VALUEFORMAT_ATTR_ID = 7;
   private static final int LABEL_ATTR_ID = 8;
   private static final int LABELFORMAT_ATTR_ID = 9;

   public void onClone(Plugin clone)
   {
      Chart cloneObj = (Chart)clone;
      cloneObj.setTitle(getTitle());
      cloneObj.setDelim(getDelim());
      cloneObj.setChartBgc(getChartBgc());
      cloneObj.setFgAlpha(getFgAlpha());
      cloneObj.setBorder(getBorder());
      cloneObj.setLegend(getLegend());
      cloneObj.setValueFormat(getValueFormat());
      cloneObj.setLabel(getLabel());
      cloneObj.setLabelFormat(getLabelFormat());
   }

   public boolean onSet(String attribute, java.util.List index,
                        UnifiedData data)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case TITLE_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setTitle(data.getStringData());
               break;

            case DELIM_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setDelim(data.getStringData());
               break;

            case CHARTBGC_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               return setChartBgc(data.getStringData());

            case FGALPHA_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setFgAlpha(data.getIntData());
               break;

            case BORDER_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setBorder(data.getIntData());
               break;

            case LEGEND_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setLegend(data.getIntData());
               break;

            case VALUEFORMAT_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setValueFormat(data.getStringData());
               break;

            case LABEL_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setLabel(data.getIntData());
               break;

            case LABELFORMAT_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (!data.isStringData())
                  return false;
               setLabelFormat(data.getStringData());
               break;
         }

         return true;
      }

      return false;
   }

   public UnifiedData onGet(String attribute, java.util.List index)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case TITLE_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getTitle());

            case DELIM_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getDelim());

            case CHARTBGC_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getChartBgc());

            case FGALPHA_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getFgAlpha());

            case BORDER_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getBorder());

            case LEGEND_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getLegend());

            case VALUEFORMAT_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getValueFormat());

            case LABEL_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getLabel());

            case LABELFORMAT_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getLabelFormat());
         }
      }

      return null;
   }

   private static HashMap attributeList = new HashMap();
   static
   {
      attributeList.put(DELIM_ATTR_NAME, new Integer(DELIM_ATTR_ID));
      attributeList.put(TITLE_ATTR_NAME, new Integer(TITLE_ATTR_ID));
      attributeList.put(CHARTBGC_ATTR_NAME, new Integer(CHARTBGC_ATTR_ID));
      attributeList.put(FGALPHA_ATTR_NAME, new Integer(FGALPHA_ATTR_ID));
      attributeList.put(BORDER_ATTR_NAME, new Integer(BORDER_ATTR_ID));
      attributeList.put(LEGEND_ATTR_NAME, new Integer(LEGEND_ATTR_ID));
      attributeList.put(VALUEFORMAT_ATTR_NAME, new Integer(VALUEFORMAT_ATTR_ID));
      attributeList.put(LABEL_ATTR_NAME, new Integer(LABEL_ATTR_ID));
      attributeList.put(LABELFORMAT_ATTR_NAME, new Integer(LABELFORMAT_ATTR_ID));
   }
}
